#!/usr/bin/python3
# -*-coding:Utf-8 -*

import sys
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
import numpy as np
import scipy
import matplotlib
# Make sure that we are using QT5
matplotlib.use('Qt5Agg')
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
import matplotlib.pyplot as plt


class Window(QWidget):
    def __init__(self, parent=None):
        super(Window, self).__init__(parent)

        self.setWindowTitle("Redresseur PD3 tout thyristor")

        # a figure instance to plot on
        self.figure1 = plt.figure()
        self.figure1.patch.set_facecolor('white')
        
        # this is the Canvas Widget that displays the `figure`
        # it takes the `figure` instance as a parameter to __init__
        self.canvas1 = FigureCanvas(self.figure1)
        # create an axis
        self.ax = self.figure1.add_subplot(211)

        # create an other axis
        self.ax2 = self.figure1.add_subplot(212)

        # this is the Navigation widget
        # it takes the Canvas widget and a parent
        self.toolbar1 = NavigationToolbar(self.canvas1, self)
        
        #  slider
        self.labelSlider = QLabel()
        self.labelSlider.setText("Retard\n à \n l'amorçage")
        
        self.slider = QSlider()
        self.MaxSlid = 180
        self.slider.setOrientation(Qt.Vertical)
        self.slider.setTickPosition(QSlider.TicksAbove)
        self.slider.setMaximum(self.MaxSlid)
        self.slider.setSingleStep(1)
        self.slider.setPageStep(10)
        self.slider.setTickInterval(10)     
        self.slider.setValue(0)
        self.slider.valueChanged.connect(self.plotSlider)
        self.slider.setToolTip('retard : souris\n ou clavier')#infobulle
        
        self.editSlider = QLineEdit()
        self.editSlider.setText("0")
        self.editSlider.setMaximumWidth(50)
        self.editSlider.textChanged.connect(self.plotLine)
        
        #  commentaires
        self.labelC = QLabel()
        self.labelC.setText("Le curseur règle le retard à l'amorçage")
        self.labelC.setGeometry(0,0,180,30)
        #  déco
        self.labelC.setStyleSheet("""
        .QLabel {
            border: 2px solid maroon;
            border-radius: 10px;
            background-color: 'white';
            text-align : center;
            }
        """)  #  text-align n'a aucun effet. d'où la ligne suivante
        self.labelC.setAlignment(Qt.AlignHCenter)

        # set the layout______________________________________________________________________________________
        layout = QVBoxLayout()  #  layout principal
        layoutFig = QHBoxLayout()  #  contient le graphe et le slider
        layoutSlid = QVBoxLayout()  #  contient le slider, son label et la zone d'édition
        layoutGraph = QVBoxLayout()  #  contient le graphe et ses menus
        
        layoutSlid.addWidget(self.labelSlider)
        layoutSlid.addWidget(self.slider)
        layoutSlid.addWidget(self.editSlider)
        
        layoutGraph.addWidget(self.canvas1)
        layoutGraph.addWidget(self.toolbar1)
        
        #  layoutFig.addWidget(self.stackFig)
        layoutFig.addLayout(layoutGraph)  #  insère layoutGraph  dans layoutFig
        layoutFig.addLayout(layoutSlid)  #  insère layoutSlid  dans layoutFig
               
        layout.addLayout(layoutFig)  #  insère layoutFig dans le layout principal
        layout.addWidget(self.labelC)
        
        self.setLayout(layout)
        #____________________________________________________________________________________________________
        
        self.n = 720#nombre de points d'échantillonage -> 2 périodes

        #  fond de courbes
        self.Te = 5.556e-5  #  période d'échantillonage=20ms/360
        temps = np.arange(self.n)*self.Te  #  array des temps
        self.Tt = [i*self.Te for i in range(720)]  #  liste des temps
        self.u1 = np.cos(2*np.pi*50.0*temps)
        self.u2 = np.cos(2*np.pi*50.0*temps-np.pi/3)
        self.u3 = np.cos(2*np.pi*50.0*temps-2*np.pi/3)
        self.u4 = np.cos(2*np.pi*50.0*temps-np.pi)
        self.u5 = np.cos(2*np.pi*50.0*temps-4*np.pi/3)
        self.u6 = np.cos(2*np.pi*50.0*temps-5*np.pi/3)
        self.v1 = np.sin(2*np.pi*50.0*temps)/np.sqrt(3)
        self.tension = []
        self.courant = []
        #  motif à décaler. Le chiffre est l'indice de u
        self.motif = scipy.linspace(1, 1, num=360)
        for j in range(5):
            for i in range(31+j*60,91+j*60): self.motif[i] = 2+j

        #  courbe nulle pour marquer l'axe
        self.nulle = scipy.linspace(0.0, 0.0, num=self.n)
        self.flagEditS = False
        self.plot()
        
    def fonction(self,f,fe):
        Phi = self.slider.value()  #  retard à l'amorçage en °
        indices = np.roll(self.motif,Phi)  #  décalage du motif de Phi degrés
        for i in range(360):
            if indices[i]==1: 
                self.tension.append(self.u1[i])
                self.courant.append(0.0)
            if indices[i]==2: 
                self.tension.append(self.u2[i])
                self.courant.append(1.0)
            if indices[i]==3: 
                self.tension.append(self.u3[i])
                self.courant.append(1.0)
            if indices[i]==4: 
                self.tension.append(self.u4[i])
                self.courant.append(0.0)
            if indices[i]==5: 
                self.tension.append(self.u5[i])
                self.courant.append(-1.0)
            if indices[i]==6: 
                self.tension.append(self.u6[i])
                self.courant.append(-1.0)
        #  assemblage des listes
        self.tension = self.tension+self.tension
        self.courant = self.courant+self.courant

    def plotSlider(self):
        if not self.flagEditS: self.editSlider.setText(str(self.slider.value()))
        self.plot()

    def plotLine(self):
        self.flagEditS = True
        self.slider.setValue(float(self.editSlider.text()))
        self.flagEditS = False

    def plot(self):
        self.tension = []
        self.courant = []
        
        f = 50.0
        self.fonction(f,1/self.Te)
        
        # discards the old graph
        self.ax.clear()
        self.ax2.clear()
        # plot data
        
        self.ax2.plot(self.Tt, self.courant, linewidth=2, label='courant réseau')
        self.ax.plot(self.Tt, self.tension, linewidth=2)
        
        self.ax2.plot(self.Tt, self.v1, linewidth=2, label='tension réseau')
        self.ax.plot(self.Tt, self.u1, 'lightgrey', linewidth=1)
        self.ax.plot(self.Tt, self.u2, 'lightgrey', linewidth=1)
        self.ax.plot(self.Tt, self.u3, 'lightgrey', linewidth=1)
        self.ax.plot(self.Tt, self.u4, 'lightgrey', linewidth=1)
        self.ax.plot(self.Tt, self.u5, 'lightgrey', linewidth=1)
        self.ax.plot(self.Tt, self.u6, 'lightgrey', linewidth=1)
        self.ax.plot(self.Tt, self.nulle, 'black')
        self.ax2.plot(self.Tt, self.nulle, 'black')
        #  légende en-dessous : mais ça sort du cadre ax2.legend(loc='upper center', bbox_to_anchor=(0.5, -0.2), shadow=True, ncol=2)
        chartBox = self.ax2.get_position()
        #  ax2.set_position([chartBox.x0, chartBox.y0, chartBox.width, chartBox.height * 0.6])
        self.ax2.legend(loc='upper left', bbox_to_anchor=(0.8, 0), shadow=True, ncol=1)

        self.ax2.set_xlabel("temps")
        self.ax2.set_ylabel("Courant réseau")
        self.ax.set_ylabel("Tension redressée")
        self.ax.set_ylim([-1.1, 1.1])
        self.ax2.set_ylim([-1.1, 1.1])

        self.ax.grid(True)
        self.ax2.grid(True)

        # refresh canvas
        self.canvas1.draw()


if __name__ == '__main__':
    app = QApplication(sys.argv)

    main = Window()
    main.show()
    app.exec_()
